@@ -26,7 +26,20 @@ DEFAULT_PARAMS = {
 }
 
 
-def attn_block(inputs, filters, strides=1, attn_type=None, attn_params={}, se_ratio=0, use_eca=False, groups=1, group_size=0, bn_after_attn=True, activation="relu", name=""):
+def attn_block(
+    inputs,
+    filters,
+    strides=1,
+    attn_type=None,
+    attn_params={},
+    se_ratio=0,
+    use_eca=False,
+    groups=1,
+    group_size=0,
+    bn_after_attn=True,
+    activation="relu",
+    name="",
+):
     nn = inputs
     if attn_params is not None and len(attn_params) != 0:
         default_attn_params = DEFAULT_PARAMS.get(attn_type, {}).copy()
@@ -36,6 +49,7 @@ def attn_block(inputs, filters, strides=1, attn_type=None, attn_params={}, se_ra
         attn_params = DEFAULT_PARAMS.get(attn_type, {})
 
     attn_act = attn_params.pop("activation", activation)
+    se_divisor = attn_params.pop("se_divisor", 8)
     if attn_type == "bot":  # mhsa_with_relative_position_embedding from botnet
         nn = attention_layers.mhsa_with_relative_position_embedding(nn, **attn_params, name=name + "mhsa_")
     elif attn_type == "halo":  # halo_attention from halonet
@@ -63,7 +77,7 @@ def attn_block(inputs, filters, strides=1, attn_type=None, attn_params={}, se_ra
         nn = batchnorm_with_activation(nn, activation=activation, zero_gamma=False, name=name)
 
     if attn_type is None and se_ratio:
-        nn = se_module(nn, se_ratio=se_ratio, activation=attn_act, name=name + "se_")
+        nn = se_module(nn, se_ratio=se_ratio, divisor=se_divisor, activation=attn_act, name=name + "se_")
 
     if attn_type is None and use_eca:
         nn = eca_module(nn, name=name + "eca_")
@@ -88,7 +102,9 @@ def conv_shortcut_branch(inputs, filters, preact=False, strides=1, shortcut_type
     return shortcut
 
 
-def deep_branch(inputs, filters, strides=1, hidden_channel_ratio=0.25, use_3x3_kernel=False, bn_after_attn=True, activation="relu", attn_block_params={}, name=""):
+def deep_branch(
+    inputs, filters, strides=1, hidden_channel_ratio=0.25, use_3x3_kernel=False, bn_after_attn=True, activation="relu", attn_block_params={}, name=""
+):
     hidden_filter = int(filters * hidden_channel_ratio)
     if use_3x3_kernel:
         nn = conv2d_no_bias(inputs, hidden_filter, 3, strides=1, padding="SAME", name=name + "deep_1_")  # Using strides=1 for not changing input shape
@@ -147,7 +163,7 @@ def aot_block(
     # print(f">>>> {inputs.shape = }, {shortcut if shortcut is None else shortcut.shape = }, {deep.shape = }, {filters = }, {strides = }")
     if preact:  # ResNetV2
         deep = drop_block(deep, drop_rate)
-        return keras.layers.Add(name=name + "add")([shortcut, deep])
+        return keras.layers.Add(name=name + "add")([shortcut, deep]) if shortcut is not None else deep  # if no shortcut
     else:
         deep = batchnorm_with_activation(deep, activation=None, zero_gamma=True, name=name + "3_")
         deep = drop_block(deep, drop_rate)
@@ -222,7 +238,7 @@ def AotNet(
     use_3x3_kernel=False,
     use_block_output_activation=True,
     stem_width=64,  # Stem params
-    stem_type=None, # ["deep", "quad", "tiered", "kernel_3x3", None]
+    stem_type=None,  # ["deep", "quad", "tiered", "kernel_3x3", None]
     quad_stem_act=False,
     stem_last_strides=1,
     stem_downsample=True,
