@@ -22,18 +22,22 @@ def mhsa_with_multi_head_relative_position_embedding(
     out_shape = cc if out_shape is None or not out_weight else out_shape
     qk_out = num_heads * key_dim
     vv_dim = out_shape // num_heads
-    # final_out_shape = (None, hh, ww, out_shape)
 
     # qkv = keras.layers.Dense(emb_dim * 3, use_bias=False, name=name and name + "qkv")(inputs)
     qkv = conv2d_no_bias(inputs, qk_out * 2 + out_shape, kernel_size=1, name=name and name + "qkv_")
-    qkv = tf.reshape(qkv, [-1, inputs.shape[1] * inputs.shape[2], qkv.shape[-1]])
-    query, key, value = tf.split(qkv, [qk_out, qk_out, out_shape], axis=-1)
-    # query = [batch, num_heads, hh * ww, key_dim]
-    query = tf.transpose(tf.reshape(query, [-1, query.shape[1], num_heads, key_dim]), [0, 2, 1, 3])
-    # key = [batch, num_heads, key_dim, hh * ww]
-    key = tf.transpose(tf.reshape(key, [-1, key.shape[1], num_heads, key_dim]), [0, 2, 3, 1])
-    # value = [batch, num_heads, hh * ww, vv_dim]
-    value = tf.transpose(tf.reshape(value, [-1, value.shape[1], num_heads, vv_dim]), [0, 2, 1, 3])
+    if vv_dim == key_dim:
+        qkv = tf.reshape(qkv, [-1, inputs.shape[1] * inputs.shape[2], 3, num_heads, key_dim])
+        query, key, value = tf.transpose(qkv, [2, 0, 3, 1, 4])
+        key = tf.transpose(key, [0, 1, 3, 2])
+    else:
+        qkv = tf.reshape(qkv, [-1, inputs.shape[1] * inputs.shape[2], qkv.shape[-1]])
+        query, key, value = tf.split(qkv, [qk_out, qk_out, out_shape], axis=-1)
+        # query = [batch, num_heads, hh * ww, key_dim]
+        query = tf.transpose(tf.reshape(query, [-1, query.shape[1], num_heads, key_dim]), [0, 2, 1, 3])
+        # key = [batch, num_heads, key_dim, hh * ww]
+        key = tf.transpose(tf.reshape(key, [-1, key.shape[1], num_heads, key_dim]), [0, 2, 3, 1])
+        # value = [batch, num_heads, hh * ww, vv_dim]
+        value = tf.transpose(tf.reshape(value, [-1, value.shape[1], num_heads, vv_dim]), [0, 2, 1, 3])
 
     # query *= qk_scale
     # [batch, num_heads, hh * ww, hh * ww]
@@ -53,7 +57,6 @@ def mhsa_with_multi_head_relative_position_embedding(
     if out_weight:
         # [batch, hh, ww, num_heads * vv_dim] * [num_heads * vv_dim, out] --> [batch, hh, ww, out]
         attention_output = keras.layers.Dense(out_shape, use_bias=out_bias, name=name and name + "output")(attention_output)
-    # attention_output.set_shape(final_out_shape)
     return attention_output
 
 
