@@ -179,6 +179,7 @@ def eca_module(inputs, gamma=2.0, beta=1.0, name=None, **kwargs):
 
 
 def drop_connect_rates_split(num_blocks, start=0.0, end=0.0):
+    """ split drop connect rate in range `(start, end)` according to `num_blocks` """
     drop_connect_rates = tf.split(tf.linspace(start, end, sum(num_blocks)), num_blocks)
     return [ii.numpy().tolist() for ii in drop_connect_rates]
 
@@ -192,25 +193,26 @@ def drop_block(inputs, drop_rate=0, name=None):
         return inputs
 
 
+@tf.keras.utils.register_keras_serializable(package="kecamCommon")
+def __anti_alias_downsample_initializer__(weight_shape, dtype="float32"):
+    import numpy as np
+
+    kernel_size, channel = weight_shape[0], weight_shape[2]
+    ww = tf.cast(np.poly1d((0.5, 0.5)) ** (kernel_size - 1), dtype)
+    ww = tf.expand_dims(ww, 0) * tf.expand_dims(ww, 1)
+    ww = tf.repeat(ww[:, :, tf.newaxis, tf.newaxis], channel, axis=-2)
+    return ww
+
+
 def anti_alias_downsample(inputs, kernel_size=3, strides=2, padding="SAME", trainable=False, name=None):
     """ DepthwiseConv2D performing anti-aliasing downsample, arxiv: https://arxiv.org/pdf/1904.11486.pdf """
-
-    def anti_alias_downsample_initializer(weight_shape, dtype="float32"):
-        import numpy as np
-
-        kernel_size, channel = weight_shape[0], weight_shape[2]
-        ww = tf.cast(np.poly1d((0.5, 0.5)) ** (kernel_size - 1), dtype)
-        ww = tf.expand_dims(ww, 0) * tf.expand_dims(ww, 1)
-        ww = tf.repeat(ww[:, :, tf.newaxis, tf.newaxis], channel, axis=-2)
-        return ww
-
     return keras.layers.DepthwiseConv2D(
         kernel_size=kernel_size,
         strides=strides,
         padding="SAME",
         use_bias=False,
         trainable=trainable,
-        depthwise_initializer=anti_alias_downsample_initializer,
+        depthwise_initializer=__anti_alias_downsample_initializer__,
         name=name and name + "anti_alias_down",
     )(inputs)
 
@@ -226,27 +228,17 @@ def make_divisible(vv, divisor=4, min_value=None):
     return new_v
 
 
-def tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="SAME", name=None):
-    """For issue https://github.com/leondgarse/keras_cv_attention_models/issues/8,
-    `tf.image.extract_patches` NOT working for TPU.
-    `overlap_1` means `1 < sizes / strides <= 2`.
-    `rates` and `name` not used, just keeping same perameters with `tf.image.extract_patche`.
+@tf.keras.utils.register_keras_serializable(package="kecamCommon")
+def __unfold_filters_initializer__(weight_shape, dtype="float32"):
+    kernel_size = weight_shape[0]
+    kernel_out = kernel_size * kernel_size
+    ww = tf.reshape(tf.eye(kernel_out), [kernel_size, kernel_size, 1, kernel_out])
+    if len(weight_shape) == 5:  # Conv3D or Conv3DTranspose
+        ww = tf.expand_dims(ww, 2)
+    return ww
 
-    input: `[batch, height, width, channel]`.
-    output: `[batch, height // strides,  width // strides, sizes * sizes * channel]`.
-
-    Examples:
-
-    >>> from keras_cv_attention_models import attention_layers
-    >>> inputs = tf.ones([1, 64, 27, 192])
-    >>> print(attention_layers.tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2).shape)
-    # (1, 32, 14, 1728)
-    """
-    # kernel_size, strides = 3, 2
-    # inputs = np.random.uniform(size=[1, 64, 28, 192])
-    kernel_size = sizes[1] if isinstance(sizes, (list, tuple)) else sizes
-    strides = strides[1] if isinstance(strides, (list, tuple)) else strides
 
+def unfold_by_conv2d(inputs, kernel_size=3, strides=2, dilation_rate=1, padding="SAME", compressed=True, name=None):
     if padding.upper() == "SAME":
         pad = kernel_size // 2
         pad_inputs = tf.pad(inputs, [[0, 0], [pad, pad], [pad, pad], [0, 0]])
@@ -254,36 +246,87 @@ def tpu_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="
         pad_inputs = inputs
 
     _, hh, ww, cc = pad_inputs.shape
-    num_patches_hh, num_patches_ww = int(tf.math.ceil(hh / strides) - 1), int(tf.math.ceil(ww / strides) - 1)
-    valid_hh, valid_ww = num_patches_hh * strides, num_patches_ww * strides
-    overlap_s = kernel_size - strides
-    temp_shape = (-1, num_patches_hh, strides, num_patches_ww, strides, cc)
-    # print(f"{ww = }, {hh = }, {cc = }, {num_patches_hh = }, {num_patches_ww = }, {valid_hh = }, {valid_ww = }, {overlap_s = }")
-    # ww = 30, hh = 66, cc = 192, num_patches_hh = 32, num_patches_ww = 14, valid_hh = 64, valid_ww = 28, overlap_s = 1
+    merge_channel = tf.transpose(pad_inputs, [0, 3, 1, 2])
+    merge_channel = tf.reshape(merge_channel, [-1, hh, ww, 1])
 
-    center = tf.reshape(pad_inputs[:, :valid_hh, :valid_ww, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    ww_overlap = tf.reshape(pad_inputs[:, :valid_hh, overlap_s : valid_ww + overlap_s, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    hh_overlap = tf.reshape(pad_inputs[:, overlap_s : valid_hh + overlap_s, :valid_ww, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    corner_overlap = tf.reshape(pad_inputs[:, overlap_s : valid_hh + overlap_s, overlap_s : valid_ww + overlap_s, :], temp_shape)  # (1, 32, 2, 14, 2, 192)
-    # print(f"{center.shape = }, {corner_overlap.shape = }")
-    # center.shape = TensorShape([1, 32, 2, 14, 2, 192]), corner_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192])
-    # print(f"{ww_overlap.shape = }, {hh_overlap.shape = }")
-    # ww_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192]), hh_overlap.shape = TensorShape([1, 32, 2, 14, 2, 192])
+    conv_rr = keras.layers.Conv2D(
+        filters=kernel_size * kernel_size,
+        kernel_size=kernel_size,
+        strides=strides,
+        dilation_rate=dilation_rate,
+        padding="VALID",
+        use_bias=False,
+        trainable=False,
+        kernel_initializer=__unfold_filters_initializer__,
+        name=name and name + "unfold_conv",
+    )(merge_channel)
 
-    center_ww = tf.concat([center, ww_overlap[:, :, :, :, -overlap_s:, :]], axis=4)  # (1, 32, 2, 14, 3, 192)
-    hh_corner = tf.concat([hh_overlap[:, :, -overlap_s:, :, :, :], corner_overlap[:, :, -overlap_s:, :, -overlap_s:, :]], axis=4)  # (1, 32, 1, 14, 3, 192)
-    out = tf.concat([center_ww, hh_corner], axis=2)  # (1, 32, 3, 14, 3, 192)
-    # print(f"{center_ww.shape = }, {hh_corner.shape = }, {out.shape = }")
-    # center_ww.shape = TensorShape([1, 32, 2, 14, 3, 192]), hh_corner.shape = TensorShape([1, 32, 1, 14, 3, 192]), out.shape = TensorShape([1, 32, 3, 14, 3, 192])
-
-    out = tf.transpose(out, [0, 1, 3, 2, 4, 5])  # [batch, height, width, kernel, kernel, channel], [1, 32, 14, 3, 3, 192]
-    # print(f"{out.shape = }")
-    # out.shape = TensorShape([1, 32, 14, 3, 3, 192])
-    return tf.reshape(out, [-1, num_patches_hh, num_patches_ww, kernel_size * kernel_size * cc])
+    out = tf.reshape(conv_rr, [-1, cc, conv_rr.shape[1], conv_rr.shape[2], kernel_size, kernel_size])
+    out = tf.transpose(out, [0, 2, 3, 4, 5, 1])  # [batch, hh, ww, kernel, kernel, channnel]
+    if compressed:
+        out = tf.reshape(out, [-1, out.shape[1], out.shape[2], kernel_size * kernel_size * pad_inputs.shape[-1]])
+    return out
 
 
-def tpu_compatible_extract_patches_overlap_1(inputs, sizes=3, strides=2, rates=1, padding="SAME", name=None):
-    if len(tf.config.experimental.list_logical_devices("TPU")) == 0:
-        return tf.image.extract_patches(inputs, sizes, strides, rates, padding, name)
+def fold_by_conv2d_transpose(patches, output_shape=None, kernel_size=3, strides=2, dilation_rate=1, padding="SAME", compressed="auto", name=None):
+    paded = kernel_size // 2 if padding else 0
+    if compressed == "auto":
+        compressed = True if len(patches.shape) == 4 else False
+
+    if compressed:
+        _, hh, ww, cc = patches.shape
+        channel = cc // kernel_size // kernel_size
+        conv_rr = tf.reshape(patches, [-1, hh, ww, kernel_size, kernel_size, channel])
     else:
-        return tpu_extract_patches_overlap_1(inputs, sizes, strides, rates, padding, name)
+        _, hh, ww, _, _, channel = patches.shape
+        conv_rr = patches
+    conv_rr = tf.transpose(conv_rr, [0, 5, 1, 2, 3, 4])  # [batch, channnel, hh, ww, kernel, kernel]
+    conv_rr = tf.reshape(conv_rr, [-1, hh, ww, kernel_size * kernel_size])
+
+    convtrans_rr = keras.layers.Conv2DTranspose(
+        filters=1,
+        kernel_size=kernel_size,
+        strides=strides,
+        dilation_rate=dilation_rate,
+        padding="VALID",
+        output_padding=paded,
+        use_bias=False,
+        trainable=False,
+        kernel_initializer=__unfold_filters_initializer__,
+        name=name and name + "fold_convtrans",
+    )(conv_rr)
+
+    out = tf.reshape(convtrans_rr[..., 0], [-1, channel, convtrans_rr.shape[1], convtrans_rr.shape[2]])
+    out = tf.transpose(out, [0, 2, 3, 1])
+    if output_shape is None:
+        output_shape = [-paded, -paded]
+    else:
+        output_shape = [output_shape[0] + paded, output_shape[1] + paded]
+    out = out[:, paded : output_shape[0], paded : output_shape[1]]
+    return out
+
+
+def tpu_compatible_extract_patches(inputs, sizes=3, strides=2, rates=1, padding="SAME", compressed=True, tpu_test=False, name=None):
+    """ Use `unfold_by_conv2d` for TPU and `extract_patches` for others, also handle `padding` and `output_shape`. """
+    if len(tf.config.experimental.list_logical_devices("TPU")) == 0 and not tpu_test:
+        if padding.upper() == "SAME":
+            kernel_size = sizes[1] if isinstance(sizes, (list, tuple)) else sizes
+            pad = kernel_size // 2
+            pad_inputs = tf.pad(inputs, [[0, 0], [pad, pad], [pad, pad], [0, 0]])
+            padding = "VALID"
+        else:
+            pad_inputs = inputs
+
+        sizes = sizes if isinstance(sizes, (list, tuple)) else [1, sizes, sizes, 1]
+        strides = strides if isinstance(strides, (list, tuple)) else [1, strides, strides, 1]
+        rates = rates if isinstance(rates, (list, tuple)) else [1, rates, rates, 1]
+        patches = tf.image.extract_patches(pad_inputs, sizes, strides, rates, padding, name)
+        if not compressed:
+            # [batch, hh, ww, kernel, kernel, channnel]
+            patches = tf.reshape(patches, [-1, patches.shape[1], patches.shape[2], sizes[1], sizes[1], inputs.shape[-1]])
+        return patches
+    else:
+        kernel_size = sizes[1] if isinstance(sizes, (list, tuple)) else sizes
+        strides = strides[1] if isinstance(strides, (list, tuple)) else strides
+        dilation_rate = rates[1] if isinstance(rates, (list, tuple)) else rates
+        return unfold_by_conv2d(inputs, kernel_size, strides, dilation_rate, padding, compressed=compressed, name=name)
